SystemOrganization addCategory: #'FileMan-Core'!
SystemOrganization addCategory: #'FileMan-Test'!

!FileDirectory methodsFor: '*FileMan-testing' stamp: 'mu 2/28/2007 00:21'!
isRoot
	^self pathName isEmpty! !

!Character methodsFor: '*fileman-testing' stamp: 'mu 3/2/2007 21:34'!
isDriveLetter
	^'ABCDEFGHIJKLMNOPQRSTUVWXYZ' includes: self asUppercase
	! !

!Character methodsFor: '*fileman-testing' stamp: 'mu 3/3/2007 09:45'!
isDriveSeparator
	^self == $:
	! !

!Character methodsFor: '*fileman-testing' stamp: 'mu 3/3/2007 19:37'!
isPathSeparator
	self = $\ ifTrue: [^true].
	self = $: ifTrue: [^true].
	self = $/ ifTrue: [^true].
	^false! !

TestCase subclass: #FmFileIOAccessorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileMan-Test'!

!FmFileIOAccessorTest methodsFor: 'private' stamp: 'mu 3/15/2007 11:10'!
defaultDirectoryPath
	^FmFileIOAccessor defaultDirectoryPath! !

!FmFileIOAccessorTest methodsFor: 'testing' stamp: 'mu 3/15/2007 11:15'!
testDirectoryExists
	"FmFileIOAccessorTest debug: #testDirectoryExists"
	| subDirString dirString |
	subDirString := 99999 atRandom asString.
	dirString := self defaultDirectoryPath, FmFileIOAccessor default slash, subDirString.

	FmFileIOAccessor default createDirectory: dirString.

	self should: [FmFileIOAccessor default directoryExists: dirString].
	self should: [FmFileIOAccessor default directoryExists: subDirString in: self defaultDirectoryPath].

	FmFileIOAccessor default deleteDirectory: dirString.

	self shouldnt: [FmFileIOAccessor default directoryExists: dirString].
	self shouldnt: [FmFileIOAccessor default directoryExists: subDirString in: self defaultDirectoryPath].! !

TestCase subclass: #FmFileManTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileMan-Test'!

!FmFileManTest commentStamp: 'mu 3/14/2007 22:27' prior: 0!
FmFileManTest suite run!

!FmFileManTest methodsFor: 'private' stamp: 'mu 11/6/2006 20:35'!
directoryEntryForTest
	^'./fmTestDir' asDirectoryEntry! !

!FmFileManTest methodsFor: 'private' stamp: 'mu 11/6/2006 20:29'!
randomFileName
	^100000 atRandom asString, '.fmtst'! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 3/15/2007 10:24'!
testAbsolutePath
	"FmFileManTest debug: #testAbsolutePath"
	| dirEntry dirEntry1 dirEntry2 dirEntry3 dirEntry4 |
	dirEntry := '/' asDirectoryEntry.
	self should: [dirEntry = ':' asDirectoryEntry].
	self should: [dirEntry = '\' asDirectoryEntry].
	
	dirEntry1 := '/temp/' asDirectoryEntry.
	self should: [dirEntry1 = ':temp' asDirectoryEntry].
	self should: [dirEntry1 = '\temp' asDirectoryEntry].

	dirEntry2 := '/temp/a' asDirectoryEntry.
	self should: [dirEntry2 = ':temp:a' asDirectoryEntry].
	self should: [dirEntry2 = '\temp\a' asDirectoryEntry].

	"Platform specific path tests"
	FmFileIOAccessor default onWindows ifTrue: [
	dirEntry3 := 'C:/temp/b' asDirectoryEntry.
	self should: [dirEntry3 = 'C:\temp\b' asDirectoryEntry].
	self should: [dirEntry3 = 'C::temp:b' asDirectoryEntry].
	].

	FmFileIOAccessor default onMac ifTrue: [
	dirEntry4 := 'Macintosh HD:tmp' asDirectoryEntry.
	self should: [dirEntry4 = 'Macintosh HD/tmp' asDirectoryEntry].
	self should: [dirEntry4 = 'Macintosh HD\tmp' asDirectoryEntry].
	].
	
	
	
	! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 11/6/2006 23:08'!
testAtPut
	"FmFileManTest debug: #testAtPut" 
	| dir bytes |
	dir _ self directoryEntryForTest.
	dir at: 'test1' put: 'Hello'.
	self should: [(dir at: 'test1') = 'Hello'].
	self should: [dir includesKey: 'test1'].

	bytes _ #(1 2 3 4 5 6) asByteArray.
	dir binaryAt: 'test2' put: bytes.
	self should: [(dir binaryAt: 'test2') = bytes].
	self should: [dir includesKey: 'test2'].

	dir removeKey: 'test1'.

	self shouldnt: [dir includesKey: 'test1'].

	dir recursiveDelete.
	self should: [dir exists not]! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 11/6/2006 23:17'!
testCopy
	"FmFileManTest debug: #testCopy" 
	| file1 file2 |
	file1 _ self randomFileName asFileEntry.
	file2 _ (file1 parent / self randomFileName) asFileEntry.

	file1 fileContents: 'This is a test'.

	self should: [file2 fileContents isEmpty].

	file1 copyTo: file2.

	self should: [file2 fileContents = 'This is a test'].

	file1 delete.
	file2 delete.
	self should: [file1 exists not].
	self should: [file2 exists not]
	
	
	! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 11/6/2006 20:33'!
testFileContents
	"FmFileManTest debug: #testFileContents" 
	| file1 file2 bytes |
	file1 _ self randomFileName asFileEntry.
	file1 fileContents: 'This is a test'.
	self should: [file1 fileContents = 'This is a test'].
	file1 delete.
	self should: [file1 exists not].

	file2 _ self randomFileName asFileEntry.
	bytes _ #(1 2 3 4 5 6) asByteArray.
	file2 fileContents: bytes.
	self should: [file2 fileContents = bytes asString].
	self should: [file2 binaryContents = bytes].
	file2 delete.
	self should: [file2 exists not]! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 11/6/2006 23:27'!
testPipe
	"FmFileManTest debug: #testPipe" 
	| reverseFilter file1 file2 file3 |

	reverseFilter := [:in :out | out nextPutAll: (in upToEnd reverse)].

	file1 _ self randomFileName asFileEntry.
	file2 _ self randomFileName asFileEntry.
	file3 _ self randomFileName asFileEntry.

	file1 fileContents: 'This is a pipe test'.

	file1 pipe: reverseFilter to: file2.

	self should: [('.' asDirectoryEntry at: file1 name) = 'This is a pipe test'].	
	self should: [(file2 fileContents) = 'tset epip a si sihT'].	
	self should: [(file3 fileContents) isEmpty].	

	file2 pipe: reverseFilter to: file3.
	self should: [(file3 fileContents) = 'This is a pipe test'].	

	file1 delete.
	file2 delete.
	file3 delete.
	self should: [file1 exists not].
	self should: [file2 exists not].
	self should: [file3 exists not]
	
	
	! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 3/15/2007 11:21'!
testRecursiveDelete
	"FmFileManTest debug: #testRecursiveDelete" 
	| dir |
	dir _ './subDir' / 'aaa\bbb' / 'ccc' / 'ddd\eee' / 'fff:ggg'.
	dir at: 'test1' put: 'RecursiveDelete!!'.
	self should: [(dir at: 'test1') = 'RecursiveDelete!!'].

	dir removeKey: 'test1'.

	self shouldnt: [(dir / 'test1') exists].

	self shouldnt: [dir exists].

	! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 11/15/2006 20:22'!
testRefresh
	"FmFileManTest debug: #testRefresh" 
	| file1 |
	file1 _ self randomFileName asFileEntry.

	file1 fileContents: '1234567890'.
	self should: [file1 fileSize = 10].

	file1 fileContents: '123'.
	self should: [file1 fileSize = 3].
	

	file1 delete.
	self should: [file1 exists not].
	! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 11/15/2006 20:19'!
testRename
	"FmFileManTest debug: #testRename" 
	| file1 |
	file1 _ self randomFileName asFileEntry.
	file1 fileContents: 'ToBeRenamed'.

	self shouldnt: [file1 name = 'newName1'].

	file1 rename: 'newName1'.

	self should: [file1 name = 'newName1'].
	self should: [file1 exists].

	self should: [file1 fileContents = 'ToBeRenamed'].

	file1 delete.
	self should: [file1 exists not].
	! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 3/3/2007 18:38'!
testRoot
	"FmFileManTest debug: #testRoot"
	| root |
	root := FmDirectoryEntry root.
	self should: [root pathComponents isEmpty].
	self should: [root = '\' asDirectoryEntry]. 
	self should: [root = ':' asDirectoryEntry]. 
	self should: [root = '/' asDirectoryEntry]. 
	
	! !

!FmFileManTest methodsFor: 'testing' stamp: 'mu 11/15/2006 20:15'!
testStream
	"FmFileManTest debug: #testStream" 
	| file1 contents formerContents allContents |
	file1 _ self randomFileName asFileEntry.
	file1 writeStreamContents: [:str | str nextPutAll: 'HELLO!!'].
	contents _ file1 readStreamContents: [:str | str upToEnd].
	self should: [contents = 'HELLO!!'].

	file1 appendStreamContents: [:str | str nextPutAll: 'AGAIN!!'].

	formerContents _ file1 readStreamContents: [:str | str upTo:$!!].
	self should: [formerContents = 'HELLO'].

	allContents _ file1 readStreamContents: [:str | str upToEnd].
	self should: [allContents = 'HELLO!!AGAIN!!'].

	file1 delete.
	self should: [file1 exists not].
	! !

Object subclass: #FmFileEntry
	instanceVariableNames: 'drive pathComponents parent name fileSize creationTime modificationTime options'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileMan-Core'!

!FmFileEntry commentStamp: 'mu 11/6/2006 20:21' prior: 0!
I represent a single file entry (including directory).

You can write data by #fileContents: , and read the data by #fileContents.

---
mu 11/6/2006 20:21!

FmFileEntry subclass: #FmDirectoryEntry
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileMan-Core'!

!FmDirectoryEntry commentStamp: 'mu 11/6/2006 20:21' prior: 0!
I represent a single file directory.
I implement various directory specific behaviors.

You can write data by #at:put: , and read the data by #at:.

---
mu 11/6/2006 20:21
!

!FmDirectoryEntry class methodsFor: 'instance creation' stamp: 'mu 3/15/2007 10:24'!
default 
	"FmDirectoryEntry default"
	^FmDirectoryEntry pathName: FmFileIOAccessor defaultDirectoryPath
	! !

!FmDirectoryEntry class methodsFor: 'instance creation' stamp: 'mu 3/14/2007 19:49'!
on: aFileDirectry 
	"FmDirectory on: FileDirectory default"
	
	| inst |
	inst := self pathName: aFileDirectry pathName.
	^ inst! !

!FmDirectoryEntry class methodsFor: 'instance creation' stamp: 'mu 6/11/2006 18:46'!
parent 
	"FmDirectory parent"
	^self default parent
	! !

!FmDirectoryEntry class methodsFor: 'instance creation' stamp: 'mu 3/14/2007 19:48'!
pathName: fullPathName

	| inst |
	inst := self new.
	inst pathName: fullPathName.
	^ inst! !

!FmDirectoryEntry class methodsFor: 'instance creation' stamp: 'mu 3/3/2007 17:13'!
root 
	"FmDirectoryEntry root"
	^'/' asDirectoryEntry! !

!FmDirectoryEntry class methodsFor: 'instance creation' stamp: 'mu 3/3/2007 10:15'!
roots
	"For the systems having notion of drives"
	"FmDirectoryEntry roots"
	^ self drives
		ifEmpty: [Array with: self root]
		ifNotEmptyDo: [:drvs | drvs
				collect: [:each | each asFileEntry]]! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:45'!
allChildrenDo: aBlock
	self childrenDo: 
		[:child | 
		aBlock value: child.
		child allChildrenDo: aBlock]! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:49'!
allDirectoriesDo: aBlock
	self directoriesDo: 
		[:child | 
		aBlock value: child.
		child allDirectoriesDo: aBlock]! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:54'!
allFilesDo: aBlock
	self childrenDo: 
		[:child | 
		child isFile ifTrue: [aBlock value: child] ifFalse: [child allFilesDo: aBlock]]! !

!FmDirectoryEntry methodsFor: '*FileMan-AddOn-archive operations' stamp: 'mu 9/21/2006 11:18'!
archive
	^self archiveMatching: [:ent | true]! !

!FmDirectoryEntry methodsFor: '*FileMan-AddOn-archive operations' stamp: 'mu 9/21/2006 11:20'!
archiveMatching: aBlock
	"Get a zip archive object filtered with matching block"
	^ZipArchive new addTree: self asFileDirectory assureExistence match: aBlock! !

!FmDirectoryEntry methodsFor: 'converting' stamp: 'mu 9/21/2006 10:21'!
asDirectoryEntry
	^self! !

!FmDirectoryEntry methodsFor: 'converting' stamp: 'mu 3/14/2007 19:06'!
asFileDirectory
	"return lagacy FileDirectory"

	^FileDirectory on: self pathName! !

!FmDirectoryEntry methodsFor: 'actions-directory' stamp: 'mu 3/13/2007 21:30'!
assureExistence
	self parent assureExistenceFileNamed: self name.
	^self! !

!FmDirectoryEntry methodsFor: 'private' stamp: 'mu 3/14/2007 22:47'!
assureExistenceFileNamed: localName
	
	localName isEmpty ifTrue: [ ^self ]. "Assumed to exist"
	(self fileAccessor fileOrDirectoryExists: localName in: self pathName) ifTrue: [^ self]. "exists"

	"otherwise check parent first and then create local dir"
	self parent ifNotNilDo: [:p | p assureExistenceFileNamed: self name].

	self fileAccessor createDirectory: (self / localName) pathName! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 3/13/2007 12:11'!
at: localFileName 
	^self contentsOf: (self fileAccessor readOnlyFileNamed: (self / localFileName) pathName)! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 7/27/2006 17:37'!
at: localFileName ifAbsent: block 
	^ [self at: localFileName]
		on: FileDoesNotExistException
		do: [:ex | block value]! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 3/14/2007 23:00'!
at: localFileName put: contents 
	self assureExistence.
	self setContentsOf: (self fileAccessor forceNewFileNamed: (self / localFileName) pathName) to: contents.
	self initChildren.
	^contents! !

!FmDirectoryEntry methodsFor: 'private' stamp: 'mu 3/15/2007 11:37'!
basicRecursiveDelete
	self directoriesDo: [:dir | dir basicRecursiveDelete].
	self filesDo: [:file | file delete].
	self delete! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 3/13/2007 12:12'!
binaryAt: localFileName 
	^self contentsOf: (self fileAccessor readOnlyFileNamed: (self / localFileName) pathName) binary! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 8/10/2006 18:47'!
binaryAt: localFileName ifAbsent: block 
	^ [self binaryAt: localFileName]
		on: FileDoesNotExistException
		do: [:ex | block value]! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 9/21/2006 10:07'!
binaryAt: localFileName put: contents 
	^self at: localFileName put: contents asByteArray! !

!FmDirectoryEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 17:37'!
children
	children ifNil: [self initChildren].
	^children! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:43'!
childrenDo: aBlock
	^self children do: aBlock! !

!FmDirectoryEntry methodsFor: 'actions-directory' stamp: 'mu 2/21/2007 09:05'!
copyTo: aDirectoryEntryOrString 
	| toDir |
	toDir := aDirectoryEntryOrString asDirectoryEntry assureExistence.

	self
		filesDo: [:file | file copyTo: (toDir / file name)].
	
	self
		directoriesDo: [:dir | | copyToDir |
			copyToDir := toDir pathName / dir name.
			dir copyTo: copyToDir]! !

!FmDirectoryEntry methodsFor: 'actions-directory' stamp: 'mu 3/13/2007 11:57'!
delete
	self fileAccessor deleteDirectory: self pathName! !

!FmDirectoryEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 17:26'!
directories
	^self children select: [:each | each isFile not]! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:42'!
directoriesDo: aBlock
	^self directories do: aBlock! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 6/18/2006 18:49'!
directoriesMatches: selectionBlock
	^self directories select: selectionBlock! !

!FmDirectoryEntry methodsFor: 'accessing' stamp: 'mu 6/20/2006 00:22'!
directory
	^self directories ifEmpty: [] ifNotEmpty: [:ds | ds first]! !

!FmDirectoryEntry methodsFor: 'testing' stamp: 'mu 3/15/2007 11:08'!
exists
	| exists |
	(self isDriveSupported and: [self pathComponents isEmpty]) ifTrue: [^true].

	exists := self fileAccessor directoryExists: self pathName. "<<-- Primitive version sometimes does not work?" 
	exists ifFalse: [exists := self fileAccessor directoryExists: self name in: self parent pathName].
	^exists! !

!FmDirectoryEntry methodsFor: 'accessing' stamp: 'mu 6/20/2006 00:22'!
file
	^self files ifEmpty: [] ifNotEmpty: [:fs | fs first]! !

!FmDirectoryEntry methodsFor: 'accessing' stamp: 'mu 3/13/2007 12:14'!
fileNames
	^self files collect: [:each | each name]! !

!FmDirectoryEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 17:27'!
files
	^self children select: [:each | each isFile]! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:40'!
filesDo: aBlock
	^self files do: aBlock! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 6/18/2006 18:48'!
filesMatches: selectionBlock
	^self files select: selectionBlock! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 3/13/2007 12:14'!
includesKey: fileName
	^self fileNames includes: fileName! !

!FmDirectoryEntry methodsFor: 'private' stamp: 'mu 3/14/2007 19:44'!
initChildren
	| dir |
	dir := self pathName.
	self exists ifFalse: [^children := #()].
	children := self fileAccessor entriesIn: dir from: self.
	^children! !

!FmDirectoryEntry methodsFor: 'testing' stamp: 'mu 6/11/2006 13:44'!
isDirectory
	^true! !

!FmDirectoryEntry methodsFor: 'testing' stamp: 'mu 6/11/2006 17:27'!
isFile
	^false! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 7/27/2006 17:35'!
keys
	^self files! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 6/18/2006 18:48'!
latestFileMatches: selectionBlock
	| entries |
	entries := self filesMatches: selectionBlock.
	entries ifEmpty: [^nil].
	^(entries sort: [:a :b | a modificationTime > b modificationTime]) first! !

!FmDirectoryEntry methodsFor: 'enumeration' stamp: 'mu 6/18/2006 18:48'!
oldestFileMatches: selectionBlock
	| entries |
	entries := self filesMatches: selectionBlock.
	entries ifEmpty: [^nil].
	^(entries sort: [:a :b | a modificationTime > b modificationTime]) last! !

!FmDirectoryEntry methodsFor: 'actions-directory' stamp: 'mu 3/15/2007 11:26'!
recursiveDelete
	self exists
		ifTrue: [[self basicRecursiveDelete]
				on: Error
				do: [:ex | self pathName asDirectoryEntry recursiveDelete]]! !

!FmDirectoryEntry methodsFor: 'initialize-release' stamp: 'mu 9/13/2006 19:37'!
refresh
	super refresh.
	children := nil! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 3/13/2007 11:55'!
removeKey: localFileName 
	self removeKey: localFileName ifAbsent: []! !

!FmDirectoryEntry methodsFor: 'dictionary-like' stamp: 'mu 3/14/2007 23:12'!
removeKey: localFileName ifAbsent: failBlock
	self fileAccessor deleteFile: (self / localFileName) pathName ifAbsent: [^failBlock value].
	self initChildren.! !

!FmDirectoryEntry methodsFor: '*FileMan-AddOn-archive operations' stamp: 'mu 9/22/2006 10:56'!
writeArchiveTo: aFileEntryOrString 
	Utilities
		informUserDuring: [:bar | 
			bar value: ('Creating archive of {1} to: {2}' translated format: {self pathName. aFileEntryOrString}).
			self archive writeTo: aFileEntryOrString asFileEntry writeStream]! !

!FmFileEntry class methodsFor: 'accessing' stamp: 'mu 3/15/2007 10:24'!
drives
	"FmFileEntry drives"
	^FmFileIOAccessor default drives! !

!FmFileEntry class methodsFor: 'instance creation' stamp: 'mu 6/15/2006 21:49'!
entry: aFileEntry
	| inst |
	inst := aFileEntry isDirectory ifTrue: [FmDirectoryEntry new] ifFalse: [FmFileEntry new].
	inst name: aFileEntry name.
	inst initValuesFrom: aFileEntry.
	^inst! !

!FmFileEntry class methodsFor: 'examples' stamp: 'mu 6/20/2006 00:24'!
example1
	"FmFileEntry example1"

	"- Create subDirectory named: 'subDir'.
	 - Put a new file named: 'file1'.
	 - Write contents 'Hello!! to that file'"

	"Traditional way"
	| subDir str |
	subDir := FileDirectory default directoryNamed: 'subDir'.
	subDir assureExistence.
	[str := subDir newFileNamed: 'file1'.
	str nextPutAll: 'Hello!!']
		ensure: [str close].

	"FileMan"

	'./subDir' asDirectoryEntry at: 'file2' put: 'Hello!!'! !

!FmFileEntry class methodsFor: 'examples' stamp: 'mu 6/20/2006 00:24'!
example2
	"FmFileEntry example2"

	"FileMan's path representation is portable"
	('./subDir' / 'aaa\bbb' / 'ccc' / 'ddd\eee' / 'fff:ggg') at: 'test1' put: 'Hello2!!'.! !

!FmFileEntry class methodsFor: 'examples' stamp: 'mu 6/20/2006 00:24'!
example3
	"FmFileEntry example3"

	"Remove 'test1' file created exapmle2"
	('./subDir' / 'aaa\bbb' / 'ccc' / 'ddd\eee' / 'fff:ggg') removeKey: 'test1'.

	"Recursive delete"
	'./subDir' asDirectoryEntry recursiveDelete! !

!FmFileEntry class methodsFor: 'examples' stamp: 'mu 6/20/2006 00:24'!
example4
	"FmFileEntry example4"

	"Collect sm directory package names"
	| packageNames |
	packageNames := './sm/cache/packages' asDirectoryEntry directories collect: [:each | each directory file name].
	packageNames inspect! !

!FmFileEntry class methodsFor: 'examples' stamp: 'mu 9/21/2006 09:57'!
example5
	"FmFileEntry example5"
	
	"Write test1 file and copy it to test2 in the parent directory"

	'./test1' fileContents: 'This is a test'.
	'./test1' asFileEntry copyTo: '../test2'.

	'../test2' fileContents inspect! !

!FmFileEntry class methodsFor: 'examples' stamp: 'mu 10/5/2006 19:58'!
example6
	"FmFileEntry example6"
	
	"test1 file contents will be written to test2 using reverse filter.
	 test2 file contents will be written to test3 again using reverse filter."

	| reverseFilter |
	'test1.txt' fileContents: 'This is a test'.
	reverseFilter := [:in :out | out nextPutAll: (in upToEnd reverse)].

	('test1.txt' asFileEntry pipe: reverseFilter to: 'test2.txt')
		pipe: reverseFilter to: 'test3.txt'.

	(#('test1.txt' 'test2.txt' 'test3.txt') collect: [:each | each fileContents]) inspect

	! !

!FmFileEntry class methodsFor: 'examples' stamp: 'mu 11/30/2006 17:00'!
example7
	"FmFileEntry example7"
	"Useful example: copy latest mcz files to releaseDir (for preparing SAR)"

	| releaseDir dir fileNames |
	releaseDir := './releasePkg' asDirectoryEntry.
	releaseDir keys isEmpty ifFalse: [releaseDir recursiveDelete].
	dir := './package-cache' asDirectoryEntry.
	fileNames := (dir keys collect: [:each | each name upTo: $.] thenSelect: [:each | each notEmpty]).
	(fileNames asSet
		collect: [:eachName | dir
				latestFileMatches: [:each | each name beginsWith: eachName]])
		do: [:eachEnt | eachEnt copyTo: releaseDir]! !

!FmFileEntry class methodsFor: 'class initialization' stamp: 'mu 9/13/2006 19:34'!
initialize
	"FmFileEntry initialize"
	Smalltalk addToStartUpList: self! !

!FmFileEntry class methodsFor: 'instance creation' stamp: 'mu 6/11/2006 17:42'!
pathComponents: comps
	| inst |
	inst := self new.
	inst pathComponents: comps.
	^inst

	! !

!FmFileEntry class methodsFor: 'instance creation' stamp: 'mu 10/26/2006 19:21'!
pathComponents: comps drive: driveString
	| inst |
	inst := self new.
	inst pathComponents: comps detectDrive: false.
	inst drive: driveString.
	^inst

	! !

!FmFileEntry class methodsFor: 'instance creation' stamp: 'mu 6/11/2006 22:21'!
pathName: aString
	| inst |
	inst := self new.
	inst pathName: aString.
	^inst

	! !

!FmFileEntry class methodsFor: 'starting up' stamp: 'mu 9/13/2006 19:35'!
startUp
	self allSubInstancesDo: [:each | each refresh]! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:47'!
allChildrenDo: aBlock
	"do nothing"! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:50'!
allDirectoriesDo: aBlock
	"do nothing"! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:51'!
allFilesDo: aBlock
	"do nothing"! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 9/27/2006 20:21'!
appendContents: aStringOrBytes 
	self
		appendStreamContents: [:str | 
			aStringOrBytes isString
				ifFalse: [str binary].
			str nextPutAll: aStringOrBytes]! !

!FmFileEntry methodsFor: 'accessing-stream' stamp: 'mu 9/27/2006 20:09'!
appendStream
	self exists
		ifFalse: [^ self writeStream].
	^ (FileStream fileNamed: self pathName) setToEnd! !

!FmFileEntry methodsFor: 'accessing-stream' stamp: 'mu 9/27/2006 20:15'!
appendStreamContents: blockWithArg 
	| stream |
	stream := self appendStream.
	[blockWithArg value: stream]
		ensure: [stream
				ifNotNilDo: [:s | s close]]! !

!FmFileEntry methodsFor: 'archive operations' stamp: 'mu 9/21/2006 12:03'!
archive
	"Get a zip archive object from thic file entry (need to be an existing compressed file)"
	^ZipArchive new readFrom: self pathName! !

!FmFileEntry methodsFor: 'converting' stamp: 'mu 6/19/2006 23:43'!
asDirectoryEntry
	^ FmDirectoryEntry pathComponents: self pathComponents drive: self drive! !

!FmFileEntry methodsFor: 'converting' stamp: 'mu 9/21/2006 10:20'!
asFileEntry
	^self! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 10/12/2006 20:18'!
assureExistence
	self exists ifTrue: [^self].
	self parent assureExistence.
	self writeStream close! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 7/30/2006 22:45'!
at: localFileName 
	^self asDirectoryEntry at: localFileName asString ifAbsent:[]! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 7/27/2006 17:33'!
at: localFileName ifAbsent: block
	^self asDirectoryEntry at: localFileName asString ifAbsent: block! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 7/19/2006 16:59'!
at: localFileName put: contents
	^self asDirectoryEntry at: localFileName asString put: contents! !

!FmFileEntry methodsFor: 'accessing-file name' stamp: 'mu 11/30/2006 17:58'!
baseName
	^self nameVersionExtension first! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 17:49'!
basicPathComponents: aCollection
	pathComponents := aCollection! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 8/10/2006 18:47'!
binaryAt: localFileName 
	^self asDirectoryEntry binaryAt: localFileName asString ifAbsent:[]! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 8/10/2006 18:47'!
binaryAt: localFileName ifAbsent: block
	^self asDirectoryEntry binaryAt: localFileName asString ifAbsent: block! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 8/10/2006 18:13'!
binaryContents
	^self contentsOf: self readStream binary! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 8/10/2006 18:15'!
binaryContents: aByteArray
	self setContentsOf: self writeStream binary to: aByteArray ! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 8/1/2006 19:24'!
children
	^self asDirectoryEntry children! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:57'!
childrenDo: aBlock
	^self asDirectoryEntry childrenDo: aBlock! !

!FmFileEntry methodsFor: 'archive operations' stamp: 'mu 9/21/2006 11:14'!
compress
	"simple gzipping"
	self readStream compressFile! !

!FmFileEntry methodsFor: 'actions-path' stamp: 'mu 6/19/2006 23:40'!
concatPathComponents: components 
	^ FmFileEntry pathComponents: (self pathComponents copy addAll: components;
			 yourself) drive: self drive! !

!FmFileEntry methodsFor: 'actions-path' stamp: 'mu 7/30/2006 14:06'!
concat: pathString
	^self concatPathComponents: pathString asString asPathComponents.! !

!FmFileEntry methodsFor: 'private' stamp: 'mu 8/10/2006 18:21'!
contentsOf: aStream
	| str conts |
	[str := aStream. 
	conts := str contents]
		ensure: [str
				ifNotNilDo: [:s | s close]].
	^ conts! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 3/13/2007 11:25'!
copyTo: aFileEntryOrString 
	| targetEntry readStr writeStr |
	targetEntry := aFileEntryOrString asFileEntry.
	targetEntry isDirectory
		ifTrue: [targetEntry := targetEntry / self name].
	[readStr := self readStream binary.
	writeStr := targetEntry writeStream binary.
	self fileAccessor copyFile: readStr toFile: writeStr]
		ensure: [readStr
				ifNotNilDo: [:r | r close].
			writeStr
				ifNotNilDo: [:w | w close]]! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 18:19'!
creationTime
	creationTime ifNil: [self initValuesFromParent].
	^creationTime! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 14:03'!
creationTime: value
	creationTime := value! !

!FmFileEntry methodsFor: 'private' stamp: 'mu 3/14/2007 19:30'!
currentDriveName
	^self fileAccessor currentDriveName! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 3/14/2007 23:11'!
delete
	self fileAccessor deleteFile: self pathName! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 8/1/2006 19:21'!
directories
	^self asDirectoryEntry directories! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:57'!
directoriesDo: aBlock
	^self asDirectoryEntry directoriesDo: aBlock! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 8/9/2006 13:55'!
directoriesMatches: selectionBlock
	^self asDirectoryEntry directoriesMatches: selectionBlock! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 8/1/2006 19:23'!
directory
	^self asDirectoryEntry directory! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 3/3/2007 22:46'!
drive
	self onUnix ifTrue: [^ drive := nil].
	drive
		ifNil: [drive := self currentDriveName].
	^ drive! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 3/2/2007 21:04'!
drives
	^self class drives! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 13:26'!
drive: aString
	drive := aString! !

!FmFileEntry methodsFor: 'private' stamp: 'mu 6/18/2006 11:29'!
ensureParent
	self pathComponents size <= 1
		ifTrue: [^ nil].
	parent := FmDirectoryEntry
				pathComponents: (self pathComponents copyFrom: 1 to: self pathComponents size - 1)
				drive: self drive.
	^ parent! !

!FmFileEntry methodsFor: 'testing' stamp: 'mu 3/14/2007 18:57'!
exists
	^self fileAccessor fileExists: self name in: self parent pathName! !

!FmFileEntry methodsFor: 'accessing-file name' stamp: 'mu 6/19/2006 23:27'!
extension
	^self nameVersionExtension last! !

!FmFileEntry methodsFor: 'archive operations' stamp: 'mu 9/22/2006 10:49'!
extractAllTo: aFileEntryOrString 
	| dir |
	dir := aFileEntryOrString asDirectoryEntry asFileDirectory.
	Utilities
		informUserDuring: [:bar | self archive
				extractAllTo: dir
				informing: bar
				overwrite: true]! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 8/1/2006 19:23'!
file
	^self asDirectoryEntry file! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 3/15/2007 10:25'!
fileAccessor
	^FmFileIOAccessor default! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 8/10/2006 18:12'!
fileContents
	"Default is text mode"
	^self textContents! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 8/10/2006 18:15'!
fileContents: aStringOrBytes 
	aStringOrBytes isString
		ifTrue: [self textContents: aStringOrBytes]
		ifFalse: [self binaryContents: aStringOrBytes]! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 18:19'!
fileSize
	fileSize ifNil: [self initValuesFromParent].
	^fileSize! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 14:07'!
fileSize: value
	fileSize := value! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 8/1/2006 19:22'!
files
	^self asDirectoryEntry files! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 9/13/2006 19:57'!
filesDo: aBlock
	^self asDirectoryEntry filesDo: aBlock! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 8/9/2006 13:54'!
filesMatches: selectionBlock
	^self asDirectoryEntry filesMatches: selectionBlock! !

!FmFileEntry methodsFor: 'private' stamp: 'mu 6/18/2006 11:45'!
getContentsOf: aStream
	| str conts |
	[str := aStream. 
	conts := str contents]
		ensure: [str
				ifNotNilDo: [:s | s close]].
	^ conts! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 11/6/2006 20:38'!
includesKey: fileName
	^self asDirectoryEntry includesKey: fileName! !

!FmFileEntry methodsFor: 'private' stamp: 'mu 6/11/2006 18:18'!
initValuesFromParent
	| targets target |
	self ensureParent.
	self parent ifNil: [^self].

	targets := self isDirectory ifTrue: [self parent directories] ifFalse: [self parent files].
	target := targets detect: [:each | each = self] ifNone:[].

	self initValuesFrom: target.
	! !

!FmFileEntry methodsFor: 'private' stamp: 'mu 6/11/2006 18:18'!
initValuesFrom: otherEntry
	otherEntry ifNil: [^self].
	self creationTime: otherEntry creationTime.
	self modificationTime: otherEntry modificationTime.
	self fileSize: otherEntry fileSize.
	! !

!FmFileEntry methodsFor: 'testing' stamp: 'mu 6/11/2006 13:45'!
isDirectory
	^false! !

!FmFileEntry methodsFor: 'testing' stamp: 'mu 3/13/2007 21:39'!
isDriveSupported
	^self fileAccessor isDriveSupported! !

!FmFileEntry methodsFor: 'testing' stamp: 'mu 6/11/2006 13:44'!
isFile
	^true! !

!FmFileEntry methodsFor: 'testing' stamp: 'mu 6/11/2006 17:27'!
isFileEntry
	^true! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 7/27/2006 17:35'!
keys
	^self asDirectoryEntry keys! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 8/9/2006 13:55'!
latestFileMatches: selectionBlock
	^self asDirectoryEntry latestFileMatches: selectionBlock! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 18:19'!
modificationTime
	modificationTime ifNil: [self initValuesFromParent].
	^modificationTime! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 14:03'!
modificationTime: value
	modificationTime := value! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 13:35'!
name
	^name! !

!FmFileEntry methodsFor: 'accessing-file name' stamp: 'mu 3/14/2007 18:03'!
nameVersionExtension
	^self fileAccessor splitNameVersionExtensionFor: self name! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/18/2006 22:05'!
name: aString 
	name := aString.
	self pathComponents
		ifNotEmpty: [self pathComponents at: self pathComponents size put: name]! !

!FmFileEntry methodsFor: 'enumeration' stamp: 'mu 8/9/2006 13:55'!
oldestFileMatches: selectionBlock
	^self asDirectoryEntry oldestFileMatches: selectionBlock! !

!FmFileEntry methodsFor: 'testing' stamp: 'mu 3/13/2007 21:38'!
onMac
	^self fileAccessor onMac! !

!FmFileEntry methodsFor: 'testing' stamp: 'mu 3/13/2007 21:38'!
onUnix
	^self fileAccessor onUnix! !

!FmFileEntry methodsFor: 'testing' stamp: 'mu 3/13/2007 21:38'!
onWindows
	^self fileAccessor onWindows! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/19/2006 23:21'!
options
	^options! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/19/2006 23:21'!
options: aCollection
	options := aCollection! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 17:41'!
parent
	parent ifNil: [parent := self ensureParent].
	^parent! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 3/13/2007 21:43'!
parent: aFmDirectory
	parent := aFmDirectory.

	self pathName: aFmDirectory pathName, self slash, self name! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 13:16'!
pathComponents
	pathComponents ifNil: [pathComponents := OrderedCollection new].
	^pathComponents! !

!FmFileEntry methodsFor: 'initialize-release' stamp: 'mu 10/26/2006 19:20'!
pathComponents: aCollection
	self pathComponents: aCollection detectDrive: true! !

!FmFileEntry methodsFor: 'initialize-release' stamp: 'mu 3/3/2007 23:34'!
pathComponents: aCollection detectDrive: detectDrive
	| tokens firstToken  |
	aCollection isEmptyOrNil ifTrue: [^pathComponents := nil].
	tokens := aCollection.
	(detectDrive and: [(firstToken := aCollection first) isDriveName])
		ifTrue: [self drive: firstToken.
			self
				basicPathComponents: (tokens copyFrom: 2 to: tokens size)]
		ifFalse: [self basicPathComponents: tokens].

	pathComponents ifNotEmpty: [self name: pathComponents last]! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 6/11/2006 13:57'!
pathName
	| str |
	str := '' writeStream.
	self printPathOn: str.
	^str contents! !

!FmFileEntry methodsFor: 'initialize-release' stamp: 'mu 3/13/2007 21:44'!
pathName: aString 
	| path tokens |
	path := aString asAbsolutePathName.
	tokens := path findTokens: self slash.
	tokens ifEmpty: [^ nil].
	(self onUnix and: [path beginsWithPathSeparator]) ifTrue: [
		^ self pathComponents: ((path beginsWith: '/') ifTrue: [tokens] ifFalse: [path asPathTokens])].
	tokens size = 1
		ifTrue: [
			path isPathSeparator ifTrue: [^self].
			self isDriveSupported
				 ifTrue: [^self setDriveNameAndPathComponentsFrom: aString].
			 ^self pathName: (FmDirectoryEntry default concat: aString) pathName].
	self pathComponents: tokens! !

!FmFileEntry methodsFor: 'actions-pipe' stamp: 'mu 10/5/2006 19:41'!
pipeRepeat: filterBlock to: aFileEntryOrString 
	^self pipeRepeat: filterBlock while: [:in :out | in atEnd not] to: aFileEntryOrString ! !

!FmFileEntry methodsFor: 'actions-pipe' stamp: 'mu 10/5/2006 19:48'!
pipeRepeat: filterBlock while: terminateBlock to: aFileEntryOrString 
	| nextEntry inStream outStream |
	nextEntry := aFileEntryOrString asFileEntry.
	[inStream := self readStream.
	outStream := nextEntry writeStream.
	[terminateBlock value: inStream value: outStream]
		whileTrue: [filterBlock value: inStream value: outStream]]
		ensure: [inStream close.
			outStream close].
	^nextEntry! !

!FmFileEntry methodsFor: 'actions-pipe' stamp: 'mu 10/5/2006 19:47'!
pipe: filterBlock to: aFileEntryOrString 
	| nextEntry inStream outStream |
	nextEntry := aFileEntryOrString asFileEntry.
	[inStream := self readStream.
	outStream := nextEntry writeStream.
	filterBlock value: inStream value: outStream]
		ensure: [inStream close.
			outStream close].
	^nextEntry
! !

!FmFileEntry methodsFor: 'printing' stamp: 'mu 2/28/2007 00:06'!
printOn: aStream 
	self printPathOn: aStream! !

!FmFileEntry methodsFor: 'printing' stamp: 'mu 3/13/2007 21:44'!
printPathOn: aStream 
	self drive
		ifNotNilDo: [:d | aStream nextPutAll: d].
	aStream nextPutAll: self slash.
	self pathComponents
		do: [:each | aStream nextPutAll: each]
		separatedBy: [aStream nextPutAll: self slash]! !

!FmFileEntry methodsFor: 'accessing-stream' stamp: 'mu 6/15/2006 23:23'!
readStream
	^ [FileStream readOnlyFileNamed: self pathName]
		on: FileDoesNotExistException
		do: [:ex | 
			self assureExistence.
			ex retry]! !

!FmFileEntry methodsFor: 'accessing-stream' stamp: 'mu 10/2/2006 23:52'!
readStreamContents: blockWithArg 
	| stream conts |
	stream := self readStream.
	[conts := blockWithArg value: stream]
		ensure: [stream
				ifNotNilDo: [:s | s close]].
	^ conts! !

!FmFileEntry methodsFor: 'initialize-release' stamp: 'mu 9/13/2006 19:36'!
refresh
	fileSize := creationTime := modificationTime := nil! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 7/19/2006 17:00'!
removeKey: localFileName 
	^self asDirectoryEntry removeKey: localFileName asString! !

!FmFileEntry methodsFor: 'dictionary-like' stamp: 'mu 9/20/2006 18:10'!
removeKey: localFileName ifAbsent: failBlock
	^self asDirectoryEntry removeKey: localFileName asString ifAbsent: failBlock! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 3/13/2007 11:52'!
rename: newName
	
	self fileAccessor rename: self pathName to: (self parent / newName) pathName.
	self name: newName! !

!FmFileEntry methodsFor: 'accessing' stamp: 'mu 3/3/2007 22:34'!
root
	^FmDirectoryEntry root! !

!FmFileEntry methodsFor: 'private' stamp: 'mu 11/6/2006 20:06'!
setContentsOf: aStream to: aStringOrBytes 
	| str |
	[str := aStream.
	aStringOrBytes isString ifFalse: [str binary].
	str nextPutAll: aStringOrBytes.
	self refresh]
		ensure: [str
				ifNotNilDo: [:s | s close]]! !

!FmFileEntry methodsFor: 'private' stamp: 'mu 3/4/2007 11:05'!
setDriveNameAndPathComponentsFrom: rawPathString 
	|  guessedDriveName |

	rawPathString first isPathSeparator
		ifTrue: [^self pathComponents: (rawPathString asPathTokens)].

	guessedDriveName := rawPathString asDriveName.
	guessedDriveName isDriveName
		ifTrue: [ | targetPathString | 
			self drive: guessedDriveName.
			targetPathString := rawPathString copyFrom: guessedDriveName size + 1 to: rawPathString size.
			^self pathComponents: targetPathString asPathTokens].

	^self pathComponents: (FmDirectoryEntry default pathComponents copyWith: rawPathString)! !

!FmFileEntry methodsFor: 'accessing-file name' stamp: 'mu 3/13/2007 21:42'!
slash
	^self fileAccessor slash! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 8/10/2006 18:12'!
textContents
	^self contentsOf: self readStream! !

!FmFileEntry methodsFor: 'actions-file' stamp: 'mu 8/10/2006 18:13'!
textContents: aString 
	self setContentsOf: self writeStream to: aString ! !

!FmFileEntry methodsFor: 'accessing-file name' stamp: 'mu 6/19/2006 23:28'!
version
	^self nameVersionExtension second! !

!FmFileEntry methodsFor: 'accessing-stream' stamp: 'mu 11/6/2006 20:07'!
writeStream
	self refresh.
	self parent exists ifFalse: [self parent assureExistence].
	^FileStream forceNewFileNamed: self pathName! !

!FmFileEntry methodsFor: 'accessing-stream' stamp: 'mu 11/6/2006 20:07'!
writeStreamConfirming
	self refresh.
	^FileStream newFileNamed: self pathName! !

!FmFileEntry methodsFor: 'accessing-stream' stamp: 'mu 9/27/2006 20:15'!
writeStreamContents: blockWithArg 
	| stream |
	stream := self writeStream.
	[blockWithArg value: stream]
		ensure: [stream
				ifNotNilDo: [:s | s close]]! !

!FmFileEntry methodsFor: 'actions-path' stamp: 'mu 6/19/2006 23:29'!
, pathString
	^self concat: pathString! !

!FmFileEntry methodsFor: 'actions-path' stamp: 'mu 6/19/2006 23:52'!
/ pathString
	^self, pathString! !

!FmFileEntry methodsFor: 'actions-rio' stamp: 'mu 3/12/2007 23:32'!
< aStringOrBytes 
	self fileContents: aStringOrBytes! !

!FmFileEntry methodsFor: 'actions-rio' stamp: 'mu 3/12/2007 23:32'!
<< aStringOrBytes 
	self appendContents: aStringOrBytes! !

!FmFileEntry methodsFor: 'comparing' stamp: 'mu 6/11/2006 18:01'!
= aFileEntry
	self class = aFileEntry class ifFalse: [^false].
	^self pathComponents = aFileEntry pathComponents! !

Object subclass: #FmFileIOAccessor
	instanceVariableNames: 'defaultDirectoryPath'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileMan-Core'!

FmFileIOAccessor class
	instanceVariableNames: 'default'!

!FmFileIOAccessor commentStamp: '<historical>' prior: 0!
I am an accessor to the low level file IO.

You can extend/rewrite me if you port FileMan to other Smalltalk dialects.

---
mu 3/13/2007 11:11!

!FmFileIOAccessor class methodsFor: 'instance creation' stamp: 'mu 3/13/2007 11:07'!
default
	default isNil ifTrue: [default := self new].
	^default! !

!FmFileIOAccessor class methodsFor: 'accessing' stamp: 'mu 3/14/2007 22:23'!
defaultDirectoryPath
	^self default defaultDirectoryPath! !

!FmFileIOAccessor class methodsFor: 'class initialization' stamp: 'mu 3/14/2007 22:19'!
initDefault
	default := nil! !

!FmFileIOAccessor class methodsFor: 'class initialization' stamp: 'mu 3/14/2007 22:26'!
initialize
	"FmFileAccessor initialize"

	Smalltalk addToStartUpList: self after: LanguageEnvironment.
	self initDefault! !

!FmFileIOAccessor class methodsFor: 'class initialization' stamp: 'mu 3/14/2007 22:20'!
obsolete
	self initDefault.
	Smalltalk removeFromStartUpList: self.
	super obsolete.! !

!FmFileIOAccessor class methodsFor: 'start up' stamp: 'mu 3/14/2007 22:21'!
startUp
	self initDefault! !

!FmFileIOAccessor methodsFor: 'private' stamp: 'mu 3/14/2007 19:18'!
bareEntriesIn: fullPath
	"FmFileIO default bareEntriesIn: ''"
	| entries index done entryArray |
	entries _ OrderedCollection new: 200.
	index _ 1.
	done _ false.
	[done] whileFalse: [
		entryArray _ self lookupEntryIn: fullPath index: index.
		#badDirectoryPath = entryArray ifTrue: [
			^(InvalidDirectoryError pathName: fullPath asSqueakPathName) signal].
		entryArray == nil
			ifTrue: [done _ true]
			ifFalse: [entries addLast: (DirectoryEntry fromArray: entryArray)].
		index _ index + 1].

	^ entries asArray collect: [:s | s convertFromSystemName].! !

!FmFileIOAccessor methodsFor: 'private' stamp: 'mu 3/15/2007 11:03'!
basicDirectoryExists: fullPathName

	| result |
	result _ self lookupEntryIn: fullPathName index: 1.
 	^result ~= #badDirectoryPath! !

!FmFileIOAccessor methodsFor: 'accessing' stamp: 'mu 3/13/2007 12:05'!
concreteStreamClass
	^FileStream concreteStream! !

!FmFileIOAccessor methodsFor: 'utilities' stamp: 'mu 3/13/2007 11:24'!
copyFile: fileStream1 toFile: fileStream2
	| buffer |
	buffer _ String new: 50000.
	[fileStream1 atEnd] whileFalse:
		[fileStream2 nextPutAll: (fileStream1 nextInto: buffer)].
! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 18:08'!
createDirectory: fullPathName
	^self primCreateDirectory: fullPathName asVmPathName! !

!FmFileIOAccessor methodsFor: 'accessing' stamp: 'mu 3/14/2007 20:16'!
currentDriveName
	^self primImagePath asDriveName! !

!FmFileIOAccessor methodsFor: 'accessing' stamp: 'mu 3/14/2007 22:56'!
defaultDirectoryPath
	"Answer the value of defaultDirectoryPath"
	defaultDirectoryPath ifNil: [self initDefaultDirectoryPath].
	^ defaultDirectoryPath! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 23:54'!
deleteDirectory: fullPathName
	^self primDeleteDirectory: fullPathName asVmPathName! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 23:12'!
deleteFile: fullPathName
	^self deleteFile: fullPathName ifAbsent: []! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 23:12'!
deleteFile: fullPathName ifAbsent: failBlock 
	^(self
			try: [self primDeleteFileNamed: fullPathName asVmPathName]
			forFileNamed: fullPathName) 
		ifFalse: [^ failBlock value]! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 23:48'!
directoryExists: fullPathName

	fullPathName = self slash ifTrue: [^true].

	^self basicDirectoryExists: fullPathName asVmPathName
	! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 19:15'!
directoryExists: localName in: directoryFullPath

	| dirNames |
	dirNames := self directoryNamesIn: directoryFullPath.

	^self isCaseSensitive 
		ifTrue:[dirNames includes: localName]
		ifFalse:[dirNames anySatisfy: [:name| name sameAs: localName]].! !

!FmFileIOAccessor methodsFor: 'utilities' stamp: 'mu 3/14/2007 19:20'!
directoryNamesIn: fullPath
	"FmFileIO default directoryNamesIn: 'D:\lang\Smalltalk\squeak\000Scallion'"
	
	^((self bareEntriesIn: fullPath) select: [:each | each isDirectory]) collect: [:each | each name]
	! !

!FmFileIOAccessor methodsFor: 'accessing' stamp: 'mu 3/14/2007 19:21'!
drives
	self onUnix ifTrue: [^#()].
	^(self bareEntriesIn: '') collect: [:each | each name]! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 19:42'!
entriesIn: directoryFullPath from: parentEntry 
	^ (self bareEntriesIn: directoryFullPath)
		collect: [:each | 
			| entry | 
			entry := FmFileEntry entry: each.
			entry parent: parentEntry.
			entry]! !

!FmFileIOAccessor methodsFor: 'utilities' stamp: 'mu 3/14/2007 22:43'!
entryNamesIn: fullPath
	"FmFileIO default entryNamesIn: 'D:\lang\Smalltalk\squeak\000Scallion'"
	
	^(self bareEntriesIn: fullPath) collect: [:each | each name]! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 22:46'!
fileExists: localName in: directoryFullPath

	| fileNames |
	fileNames := self fileNamesIn: directoryFullPath.
	^self isCaseSensitive 
		ifTrue:[fileNames includes: localName]
		ifFalse:[fileNames anySatisfy: [:name| name sameAs: localName]].! !

!FmFileIOAccessor methodsFor: 'utilities' stamp: 'mu 3/14/2007 22:46'!
fileNamesIn: fullPath
	"FmFileAccessor default fileNamesIn: 'D:\lang\Smalltalk\squeak\000Scallion'"
	
	^((self bareEntriesIn: fullPath) reject: [:each | each isDirectory]) collect: [:each | each name]! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 22:44'!
fileOrDirectoryExists: localName in: directoryFullPath

	| entryNames |
	entryNames := self entryNamesIn: directoryFullPath.

	^self isCaseSensitive 
		ifTrue:[entryNames includes: localName]
		ifFalse:[entryNames anySatisfy: [:name| name sameAs: localName]].! !

!FmFileIOAccessor methodsFor: 'file stream creation' stamp: 'mu 3/13/2007 12:09'!
forceNewFileNamed: fullPathName
	"Open the file with the given name in this directory for writing.  If it already exists, delete it first without asking."

	^ self concreteStreamClass forceNewFileNamed: fullPathName
! !

!FmFileIOAccessor methodsFor: 'private' stamp: 'mu 3/14/2007 22:05'!
imageDirectoryPath
	| imagePath lastIndex |
	imagePath := self primImagePath.
	lastIndex := imagePath lastIndexOf: self pathNameDelimiter.
	^ imagePath copyFrom: 1 to: lastIndex! !

!FmFileIOAccessor methodsFor: 'initialize-release' stamp: 'mu 3/14/2007 22:05'!
initDefaultDirectoryPath
	| dir |
	dir := Preferences startInUntrustedDirectory
				ifTrue: [SecurityManager default untrustedUserDirectory]
				ifFalse: [self imageDirectoryPath].
	self initDefaultDirectoryPathWith: dir! !

!FmFileIOAccessor methodsFor: 'initialize-release' stamp: 'mu 3/14/2007 20:32'!
initDefaultDirectoryPathWith: barePath
	| dirName |
	dirName := (FilePath pathName: barePath) asSqueakPathName asSqueakPathName.

	[dirName endsWith: self slash] whileTrue:[
		dirName := dirName copyFrom: 1 to: dirName size - self slash size.
	].
	defaultDirectoryPath := dirName! !

!FmFileIOAccessor methodsFor: 'initialize-release' stamp: 'mu 3/14/2007 20:23'!
initialize
	super initialize.
	defaultDirectoryPath := nil! !

!FmFileIOAccessor methodsFor: 'testing' stamp: 'mu 3/14/2007 22:07'!
isCaseSensitive
	"FmFileIO default isCaseSensitive"
	^self onUnix! !

!FmFileIOAccessor methodsFor: 'testing' stamp: 'mu 3/13/2007 21:40'!
isDriveSupported
	^self onWindows or: [self onMac]! !

!FmFileIOAccessor methodsFor: 'private' stamp: 'mu 3/14/2007 18:08'!
lookupEntryIn: fullPathName index: index
	
	^self primLookupEntryIn: fullPathName asVmPathName index: index! !

!FmFileIOAccessor methodsFor: 'testing' stamp: 'mu 3/14/2007 20:08'!
onMac
	^self pathNameDelimiter = $:! !

!FmFileIOAccessor methodsFor: 'testing' stamp: 'mu 3/14/2007 20:08'!
onUnix
	^self pathNameDelimiter = $/! !

!FmFileIOAccessor methodsFor: 'testing' stamp: 'mu 3/14/2007 20:08'!
onWindows
	^self pathNameDelimiter = $\! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 20:09'!
pathNameDelimiter
	^self primPathNameDelimiter! !

!FmFileIOAccessor methodsFor: 'primitives' stamp: 'mu 3/13/2007 00:11'!
primCreateDirectory: fullPath
	"Create a directory named by the given path. Fail if the path is bad or if a file or directory by that name already exists."

 	<primitive: 'primitiveDirectoryCreate' module: 'FilePlugin'>
	self primitiveFailed
! !

!FmFileIOAccessor methodsFor: 'primitives' stamp: 'mu 3/13/2007 00:11'!
primDeleteDirectory: fullPath
	"Delete the directory named by the given path. Fail if the path is bad or if a directory by that name does not exist."

 	<primitive: 'primitiveDirectoryDelete' module: 'FilePlugin'>
	self primitiveFailed
! !

!FmFileIOAccessor methodsFor: 'primitives' stamp: 'mu 3/13/2007 00:11'!
primDeleteFileNamed: aFileName
	"Delete the file of the given name. Return self if the primitive succeeds, nil otherwise."

	<primitive: 'primitiveFileDelete' module: 'FilePlugin'>
	^ nil
! !

!FmFileIOAccessor methodsFor: 'primitives' stamp: 'mu 3/14/2007 19:27'!
primImagePath

	<primitive: 121>
	self primitiveFailed! !

!FmFileIOAccessor methodsFor: 'primitives' stamp: 'mu 3/13/2007 00:12'!
primLookupEntryIn: fullPath index: index
	"Look up the index-th entry of the directory with the given fully-qualified path (i.e., starting from the root of the file hierarchy) and return an array containing:

	<name> <creationTime> <modificationTime> <dirFlag> <fileSize>

	The empty string enumerates the top-level files or drives. (For example, on Unix, the empty path enumerates the contents of '/'. On Macs and PCs, it enumerates the mounted volumes/drives.)

	The creation and modification times are in seconds since the start of the Smalltalk time epoch. DirFlag is true if the entry is a directory. FileSize the file size in bytes or zero for directories. The primitive returns nil when index is past the end of the directory. It fails if the given path is bad."

 	<primitive: 'primitiveDirectoryLookup' module: 'FilePlugin'>
	^ #badDirectoryPath

! !

!FmFileIOAccessor methodsFor: 'primitives' stamp: 'mu 3/14/2007 19:51'!
primPathNameDelimiter
	"Return the path delimiter for the underlying platform's file system."

 	<primitive: 'primitiveDirectoryDelimitor' module: 'FilePlugin'>
	self primitiveFailed
! !

!FmFileIOAccessor methodsFor: 'primitives' stamp: 'mu 3/13/2007 00:12'!
primRename: oldFileFullName to: newFileFullName 
	"Rename the file of the given name to the new name. Fail if there is no file of the old name or if there is an existing file with the new name.
	Changed to return nil instead of failing ar 3/21/98 18:04"

	<primitive: 'primitiveFileRename' module: 'FilePlugin'>
	^nil! !

!FmFileIOAccessor methodsFor: 'file stream creation' stamp: 'mu 3/13/2007 12:05'!
readOnlyFileNamed: fullPathName
	"Open the existing file with the given name in this directory for read-only access."

	^ self concreteStreamClass readOnlyFileNamed: fullPathName
! !

!FmFileIOAccessor methodsFor: 'actions' stamp: 'mu 3/14/2007 23:47'!
rename: oldFileFullName to: newFileFullName 
	| selection |
	(self try: [self primRename: oldFileFullName asVmPathName to: newFileFullName asVmPathName]
			forFileNamed: oldFileFullName) ifTrue: [^ self].

	(self fileExists: oldFileFullName) ifFalse: [^ self error: 'Attempt to rename a non-existent file'].
	(self fileExists: newFileFullName)
		ifTrue: [selection := (PopUpMenu labels: 'delete old version
cancel')
						startUpWithCaption: 'Trying to rename a file to be
' , newFileFullName , '
and it already exists.'.
			selection = 1
				ifTrue: [self deleteFile: newFileFullName.
					^ self rename: oldFileFullName toBe: newFileFullName]].
	^ self error: 'Failed to rename file'! !

!FmFileIOAccessor methodsFor: 'accessing' stamp: 'mu 3/14/2007 20:09'!
slash
	^self pathNameDelimiter asString! !

!FmFileIOAccessor methodsFor: 'utilities' stamp: 'mu 3/14/2007 18:03'!
splitNameVersionExtensionFor: fileName
	" answer an array with the root name, version # and extension.
	See comment in nextSequentialNameFor: for more details"

	| baseName version extension i j |

	baseName _ self class baseNameFor: fileName.
	extension _ self class extensionFor: fileName.
	i _ j _ baseName findLast: [:c | c isDigit not].
	i = 0
		ifTrue: [version _ 0]
		ifFalse:
			[(baseName at: i) = $.
				ifTrue:
					[version _ (baseName copyFrom: i+1 to: baseName size) asNumber.
					j _ j - 1]
				ifFalse: [version _ 0].
			baseName _ baseName copyFrom: 1 to: j].
	^ Array with: baseName with: version with: extension! !

!FmFileIOAccessor methodsFor: 'private' stamp: 'mu 3/14/2007 23:45'!
try: execBlock forFileNamed: fullName

	"If fail, return nil"

	^ (self concreteStreamClass retryWithGC: execBlock until: [:result | result notNil] forFileNamed: fullName) notNil! !

!Object methodsFor: '*fileman-testing' stamp: 'mu 10/12/2006 20:20'!
asFileEntry
	^self asString asFileEntry! !

!Object methodsFor: '*fileman-testing' stamp: 'mu 6/11/2006 17:28'!
isFileEntry
	^false! !

!String methodsFor: '*fileman-converting' stamp: 'mu 3/4/2007 11:04'!
asAbsolutePathName

	"Not complete, but in most cases it is OK"

	| tokens curDir childPath |

	self isRelativePathName ifFalse: [^self].

	self = '.' ifTrue: [^FileDirectory default pathName].
	self = '..' ifTrue: [^FileDirectory default containingDirectory pathName].

	tokens := self asPathTokens.

	curDir := FileDirectory default.
	tokens reverseDo: [:each |
		each = '..' ifTrue: [curDir := curDir containingDirectory]
	].

	tokens removeAllSuchThat: [:each | #('.' '..') includes: each ].
	childPath := WriteStream on: ''.
	tokens do: [:each | childPath nextPutAll: each]
					separatedBy: [childPath nextPutAll: FileDirectory slash].
		
	^curDir pathName, FileDirectory slash, childPath contents! !

!String methodsFor: '*fileman-converting' stamp: 'mu 6/15/2006 21:48'!
asDirectoryEntry
	^FmDirectoryEntry pathName: self! !

!String methodsFor: '*fileman-converting' stamp: 'mu 3/15/2007 10:24'!
asDriveName
	(FmFileIOAccessor default onWindows and: [self beginsWithDriveLetter]) ifTrue: [
			^self copyFrom: 1 to: 2.
	].

	(FmFileIOAccessor default onMac) ifTrue: [ 
			^self upToFirstPathSeparator
	].

! !

!String methodsFor: '*fileman-converting' stamp: 'mu 6/11/2006 22:42'!
asFileEntry
	self isRelativeMark ifTrue: [^self asDirectoryEntry].
	^FmFileEntry pathName: self! !

!String methodsFor: '*fileman-converting' stamp: 'mu 3/4/2007 11:04'!
asPathComponents
	| tokens |

	self isRelativePathName ifTrue: [self error: 'relative form is invaild!!'].

	tokens := self asPathTokens.
	^tokens! !

!String methodsFor: '*fileman-converting' stamp: 'mu 3/4/2007 11:04'!
asPathTokens
	| tokens |
	tokens := self findTokens: '\:/'.
	^tokens! !

!String methodsFor: '*fileman-testing' stamp: 'mu 3/3/2007 09:46'!
beginsWithDriveLetter
	self size < 2 ifTrue: [^false].
	^self first isDriveLetter and: [self second isDriveSeparator ]! !

!String methodsFor: '*fileman-testing' stamp: 'mu 3/4/2007 10:22'!
beginsWithPathSeparator
	self ifEmpty: [^false].
	^self first isPathSeparator! !

!String methodsFor: '*fileman-actions' stamp: 'mu 6/11/2006 22:12'!
fileContents
	^self asFileEntry fileContents! !

!String methodsFor: '*fileman-actions' stamp: 'mu 6/11/2006 22:19'!
fileContents: aString
	self asFileEntry fileContents: aString.
	^aString! !

!String methodsFor: '*fileman-accessing' stamp: 'mu 3/3/2007 19:38'!
indexOfFirstPathSeparator
	self
		doWithIndex: [:each :idx | each isPathSeparator
				ifTrue: [^ idx]].
	^ 0! !

!String methodsFor: '*fileman-testing' stamp: 'mu 3/2/2007 21:20'!
isAbsolutePathName
	self ifEmpty: [^ false].
	('\:/' includes: self first) ifTrue: [^ true].
	(FmFileEntry drives
			anySatisfy: [:each | (self asUppercase beginsWith: each)
					and: ['\:/' includes: (self atWrap: each size + 1)]]) ifTrue: [^ true].
	^ false! !

!String methodsFor: '*fileman-testing' stamp: 'mu 3/15/2007 10:24'!
isDriveName
	FmFileIOAccessor default onWindows
		ifTrue: [^ (self size between: 2 and: 3)
				and: [(self second = $:)
						and: [self first asCharacter isDriveLetter]]].

	FmFileIOAccessor default onMac ifTrue: [^FileDirectory root directoryNames includes: self].

	^false! !

!String methodsFor: '*fileman-testing' stamp: 'mu 3/2/2007 22:05'!
isPathSeparator
	self = '\' ifTrue: [^true].
	self = ':' ifTrue: [^true].
	self = '/' ifTrue: [^true].
	^false! !

!String methodsFor: '*fileman-testing' stamp: 'mu 6/11/2006 22:41'!
isRelativeMark
	
	self = '.' ifTrue: [^true].
	self = '..' ifTrue: [^true].

	^false! !

!String methodsFor: '*fileman-testing' stamp: 'mu 3/4/2007 10:46'!
isRelativePathName
	self ifEmpty: [^false].
	self isRelativeMark ifTrue: [^true].
	^#('.' '..') includes: self upToFirstPathSeparator ! !

!String methodsFor: '*fileman-accessing' stamp: 'mu 3/4/2007 10:45'!
upToFirstPathSeparator
	| index |
	self ifEmpty: [^self].
	index := self indexOfFirstPathSeparator.
	index = 0 ifTrue: [^self].
	^self copyFrom: 1 to: index - 1
	! !

!String methodsFor: '*fileman-converting' stamp: 'mu 6/19/2006 23:50'!
/ aString
	^self asDirectoryEntry concat: aString! !
FmFileEntry initialize!
FmFileIOAccessor initialize!
